comment %
constants:
flopper_top and flopper_bottom labels are used to determine the total
amount of code to move above 512k
routines grow up from 512k toward 640
disk_start points to the 1st image's  size+params+image data
%



flopper_top:

; the following structure data will always be located on a paragraph
; boundry, so it can always be accessed at offset 0 of whatever CS we're
; running in.  This is important, so don't add any code between flopper_top
; and flopper_data structure

flopper_data struc
        flagbyte        db      DISK_PROTECTED  ; misc flags
        vidmode         db      ?               ; current video mode
        vidpage         db      ?               ; current video page
        cursorpos       dw      ?               ; current cusor position
        delayAmount     db      ?               ; amount of system delay
flopper_data ends

flopperdata flopper_data <>

;
;        O L D     I N T E R R U P T     V E C T O R S
;
realInt13:
        db      0eah            ; far jmp into interrupt code
        dd      ?
realInt15:
        pushf
        db      09ah            ; far call into interrupt code
        dd      ?
        ret
realInt5:                       ; far jmp to interrupt code
        db      0eah
        dd      ?
;
;        I N T E R R U P T        1 3 h       C O D E
;
INT13:

    ;    _mcls
    ;    _mshowregs



        cmp     dl, 0                   ; A: drive?
        jnz     real_int13              ; no, skip flopper support.  

        cmp     ah, RESET_DRIVE         ; Reset disk subsystem?
        jz      verify

        cmp     ah, VERIFY_SECTORS      ; verify sectors?
        jz      verify

        cmp     ah, WRITE_SECTORS       ; writing sectors?
        jz      write

        cmp     ah, READ_SECTORS        ;
        jnz     real_int13              ; send everything else to BIOS

read:
        push    di
        call    disk_changed?           ; has floppy been removed on us?
        jc      exit_int13
        call    select_disk             ; set up EDI to point to start of disk
        call    valid_sector?           ; check to see if sector is available
        jc      exit_int13
        call    sector2offset           ; translate parameters to offset
        call    readSectors             ; fill buffer with required data
        jmp     exit_int13

write:
        ; could do all the setup work then figure out if we're reading or
        ; writing to memory to optimize code.  maybe next time.

        push    di
        call    disk_changed?           ; has floppy been removed on us?
        jc      exit_int13
        call    disk_writeprotected?    ; can we write?
        jc      exit_int13
        call    select_disk             ; set up EDI to point to start of disk
        call    valid_sector?           ; check to see if sector is available
        jc      exit_int13
        call    sector2offset           ; set up target buffer
        call    writeSectors            ; write data to image
        call    update_writeflag        ; set flag for exit
        jmp     exit_int13

verify:
	call	floppyAccessDelay	; simulate heads seeking
	call	floppyAccessDelay	; simulate heads reading

	push    di
        clc                             ; uh, yep, everything "verified" ok!
        mov     ah, NO_DISK_ERROR

exit_int13:
        pop     di                      
        sti
        retf 2

real_int13:

        jmp     realInt13               ; call real INT13
;----------------------------------------------------------------------------
;procedure: sect2offset
;purpose: converts an int 13 sector request to an offset in the image file
;procedure wants: standard INT 13h call format (CH=track,CL=sector#,DH=head)
;                 EDI=pointer to disk's parameter table      
;procedure returns: EDI=offset in file where "sector" begins
;procedure destroys: just about everything.  
;============================================================================
sector2offset proc near

    ;algoryrythym:  offset=track*SectorsPerTrack*Heads
    ;                      +head*SectorsPerTrack
    ;                      +sectorNumber
    ;offset=offset*512, assuming 512 byte sectors.


        push    ax
        push    bx
        push    cx
        push    dx
        push    si

        ; calculate sectorsPerTrack * Heads

        push    cx
        mov     al, byte ptr gs:[edi+1]         ; read SPT
        mov     cl, byte ptr gs:[edi]           ; read heads
        dec     cl
        shl     al, cl                          ; multiply by heads
        pop     cx

        ; multiply * track number (CH)

        push    cx
        xor     ah, ah
        xor     cl, cl
        xchg    cl, ch
        mul     cl
        pop     cx

        ; AX=track#*sector/track*heads

        
        shl     eax, 16                         ; save AX
        mov     al, byte ptr gs:[edi+1]         ; read SPT again
        xor     dl, dl
        xchg    dh, dl
        mul     dl      

        ; AX=head*sector/track


        dec     cl                              ;start with 0 based sector
        xor     ch, ch
        add     cx, ax
        shr     eax, 16                         ; restore ax
        add     ax, cx


        ; AX is now equal to the sector offset in the file


        add     edi, 4                          ; move beyond disk params

        mov     dx, SECTOR_SIZE
        xor     cx, cx
        xor     bx, bx
        mul     dx                              ; multiply sector size

        mov     si, dx                          ; move offset into EDI
        shl     esi,16
        mov     si, ax

        add     edi, esi

        pop     si
        pop     dx
        pop     cx
        pop     bx
        pop     ax
        ret
sector2offset endp


;----------------------------------------------------------------------------
;procedure: read_sectors
;purpose: reads a "sector" from image file
;procedure wants: INT 13 stuff , head,sector,track,etc ES:BX=buffer for data
;               EDI points to disk's parameter table.
;procedure returns: buffer filled with data, carry flag clear if no error
;procedure destroys: EDI
;notes:  
;============================================================================
readSectors proc near

        ; AL should be the number of sectors to read
        ; ES:BX = pointer to target buffer
        ; EDI = pointer of source buffer
        ; copy SECTOR_SIZE bytes, AL times

        push    ax
        push    bx
        push    cx
        push    edx

        xor     ah, ah
        mov     cx, SECTOR_SIZE / 4
        mul     cx

        mov     cx, ax        

	call	floppyAccessDelay		; simulate heads seeking
readloop:
	call	floppyAccessDelay		; simulate the read delay
						; from floppy
	mov     edx, gs:[edi]
        mov     es:[bx], edx
        add     edi, 4
        add     bx, 4
        loop    readloop

        pop     edx
        pop     cx
        pop     bx
        pop     ax
        mov     ah, NO_DISK_ERROR                  ; return no errors
        clc
        ret

readSectors endp

;----------------------------------------------------------------------------
;procedure: write_sectors
;purpose: writes data to a "sector" in image file
;procedure wants: INT 13 stuff , head,sector,track,etc ES:BX=buffer for data
;               EDI points to disk's parameter table.
;procedure returns: buffer filled with data, carry flag clear if no error
;procedure destroys: EDI
;notes:  
;============================================================================
writeSectors proc near

        ; AL should be the number of sectors to read
        ; ES:BX = pointer to source buffer
        ; EDI = pointer of target buffer
        ; copy SECTOR_SIZE bytes, AL times

        push    ax
        push    bx
        push    cx
        push    edx

        xor     ah, ah
        mov     cx, SECTOR_SIZE / 4
        mul     cx

        mov     cx, ax        

	call	floppyAccessDelay		; simulate heads seeking
						
writeloop:
	call	floppyAccessDelay		; simulate the write delay
						; from floppy
	mov     edx, es:[bx]
        mov     gs:[edi], edx
        add     edi, 4
        add     bx, 4
        loop    writeloop

        pop     edx
        pop     cx
        pop     bx
        pop     ax
        mov     ah, NO_DISK_ERROR                  ; return no errors
        clc
        ret

writeSectors endp


;------------------------------------------------------------------------
; simulate a floppy drive by delaying for head access and sector reading.
; copying data from memory is MUCH faster than off a floppy, so let's wait
; around some.
; input: none
; output: none.  
; this is an idle delay.  Value is based on a floppy seek and read on a 4.77
; Mhz machine.
floppyAccessDelay	proc
        push    si
        xor     si, si
        test    cs:[si].flagbyte, NOFLOPPY      ; no floppy delay?
        pop     si
        jnz     @f

	push	cx
        mov     cx, 25                         ; wait 25mS
	call	delay1ms
	pop	cx
@@:
	ret
floppyAccessDelay	endp



;------------------------------------------------------------------------
; reads current disk # and points EDI to start of diskette image -4.
;
select_disk     proc    near
        mov     edi, DISK_START
        add     edi, 4                  ; point beyond disk size
        ret
select_disk endp


;------------------------------------------------------------------------
; check if the selected sector is beyond the disk.  Ie, reading sector
; 10 on a 9 sector/track disk.
valid_sector?   proc    near

	push    ax
        sub     al, 1                   ; make 0 based
        jc      sector_not_found        ; 
        add     al, cl                  ; add starting sector with # of sec.
        cmp     al, byte ptr gs:[EDI+1] ; bigger than sectors/track?
        pop     ax
        ja      sector_not_found        ; bail!
        push    es
        push    bx
        les     bx, gs:[(1Eh*4)]  ; load disk param table into es:bx
        cmp     byte ptr es:[bx+3], 2   ; sector size
        pop     bx
        pop     es
        jne     sector_not_found
        clc
        ret
sector_not_found:
        mov     ah, SECTOR_NOT_FOUND_ERROR
        xor     al, al
        stc
        ret
valid_sector?   endp

disk_writeprotected? proc near
; set carry if diskette is write protected.
        push    si
        xor     si, si
        test    cs:[si].flagbyte, DISK_PROTECTED
        clc
        jz      ok2write
        stc     
        mov     ah, DISK_WP_ERROR
ok2write:
        pop     si
        ret
disk_writeprotected? endp


update_writeflag proc near
; set flag byte to show image changed
        push    si
        xor     si, si
        or      cs:[si].flagbyte, DISK_WRITTEN
        pop     si
        ret
update_writeflag endp



disk_changed? proc near
;returns carry set and error status if disk has changed.
        push    si
        xor     si, si
        clc                     ; assume no
        test    cs:[si].flagbyte, DISK_CHANGE
        jz      nochange
        mov     ah, DISK_CHANGED_ERROR
        and     cs:[si].flagbyte, NOT DISK_CHANGE   ; turn off status          
        stc     
nochange:
        pop     si
        ret
disk_changed? endp


;
;        I N T E R R U P T        1 5 h       C O D E
;
; intercept INT 15, ah=4f to snag keyboard stuff. called by INT 9
;

INT15:
        call    realInt15



        pushf
        cmp     al, F12_SCAN_CODE
        jnz     exit_int15
        popf


      ;  call    saveregs                 ; future, never restores regs yet.


    ;    out     80h, al





        ; prepare to enter chooser
        ; the video page, mode and cursor position need to be saved before
        ; memory is swapped out.
        ; these items need to be restored 
      
        push    ax
        push    bx
        push    cx
        push    dx
        push    si
        xor     si, si
        mov     ah, 0fh
        int     10h
        
        mov     cs:[si].vidmode, al
        mov     cs:[si].vidpage, bh

        mov     ah, 03
        int     10h                                     ; get cursor position
        mov     cs:[si].cursorpos, dx       ; save it


        pop     si
        pop     dx                       
        pop     cx
        pop     bx
        pop     ax



        _swap512k


        ; jump to chooser
        push    bx
        push    ax

        push    si
        xor     si, si
        mov     al, cs:[si].flagbyte    ; retrieve status
	mov	ah, cs:[si].delayAmount ; retrieve delay counter
	mov     bl, cs:[si].vidmode
        mov     bh, cs:[si].vidpage
        pop     si

        int     03fh                    ; invoke chooser

        push    si
        xor     si, si
        mov     cs:[si].flagbyte, al    ; return status
	mov	cs:[si].delayAmount, ah ; return new delay amount
	pop     si

        pop     ax
        pop     bx



        push    ax
        push    bx
        push    cx
        push    dx
        push    si
        xor     si, si


        mov     bh, cs:[si].vidpage   ; bl=mode bh=page
        mov     dx, cs:[si].cursorpos ; get cursor position
        mov     ah, 02
        int     10h                      ; set cursor position


        pop     si
        pop     dx                       
        pop     cx
        pop     bx
        pop     ax


    ;    pop     bx

        _swap512k
        sti



       ; call    restoreregs

        pushf
exit_int15:
        popf
        iret
;---------------------------------------------------------------------------
;saveregs.  copies all registers into high memory for later restoring
;without a stack.
;chooser MUST restore any 32 bit registers used!

saveregs proc near          
        push    edi
        push    esi
        rol     esi, 16
        mov     di, si

        mov     edi, REGS_START
        mov     word ptr gs:[edi], ax
        add     edi, 2
        mov     word ptr gs:[edi], bx
        add     edi, 2
        mov     word ptr gs:[edi], cx
        add     edi, 2
        mov     word ptr gs:[edi], dx
        add     edi, 2
        mov     word ptr gs:[edi], cs
        add     edi, 2
        mov     word ptr gs:[edi], ds
        add     edi, 2
        mov     word ptr gs:[edi], es
        add     edi, 2
        mov     word ptr gs:[edi], ss
        add     edi, 2
        mov     word ptr gs:[edi], sp
        add     edi, 2
        mov     word ptr gs:[edi], bp
        add     edi, 2
        mov     word ptr gs:[edi], si           ; save di
        ror     esi, 16
        add     edi, 2
        mov     word ptr gs:[edi], si           ; save si
        add     edi, 2

        push    ax
        pushf
        pop     ax
        mov     word ptr gs:[edi], ax           ; save flags
        pop     ax


        pop     esi
        pop     edi
        ret

saveregs endp


;
;        I N T E R R U P T        5       C O D E 
;
; screenshot.
INT5:

        ; copy palette byte from 40:65 and put it into b800:3ffff
        ; b800:3fff is not used on the screen, so it is safe to place a
        ; byte here without modifying screen contents.

        push    ax
        push    es
        push    di

        mov     al, gs:[465h]

        mov     di, 0b800h
        mov     es, di
        mov     di, 3fffh

        stosb

        pop     di
        pop     es
        pop     ax
       
        _swap512k

        ; jump to chooser
        push    si
        xor     si, si
	mov	ah, cs:[si].delayAmount	; retrieve delay count
	mov     al, cs:[si].flagbyte    ; retrieve status
        pop     si
        or      al, SCREENSHOT

        int     03fh                    ; invoke chooser

        push    si
        xor     si, si
        mov     cs:[si].flagbyte, al    ; return status
	mov	cs:[si].delayAmount, ah	; return delay count
	pop     si


      ; do we need this stuff here?
        
        ; signal end of interrupt
        push    ax
        mov     al, 20h
        out     20h, al
        out     0a0h, al
        out     20h, al

        int     9
      ;
        pop     ax
        _swap512k

        iret

;
;        I N T E R R U P T        70h       C O D E 
;
INT70:
	push	ax
	push	si
	push	cx
	xor	si, si
	movzx	cx, byte ptr cs:[si].delayAmount
	
@@:
	call	delay1_4ms		; delay 1/250 of a second
   	loop	@b
	pop	cx
	pop	si


exit_int70:
        cli                       
        mov     al, 0ch                ; read cmos status register 
        out     70h, al                ; which signals RTC that it can generate
        in      al, 71h                ; INT 70 again
        sti

        mov     al, 20h                ; signal end of interrupt
        out     20h, al
        out     0a0h, al               
        out     20h, al
        pop     ax
        iret



;----------------------------------------------------------------------------
;       delay1ms - Delay for 1 millasecond.
;		    1mS = 1000us
;       Entry:
;         None
;       Exit:
;	  None
;
;       Modified:
;         None
;
PORTB			EQU	061h
  REFRESH_STATUS	EQU	010h		; Refresh signal status
delay1ms PROC NEAR
        push    ax 
        push    cx
        mov     cx, 66			; close enough.
	in	al,PORTB
	and	al,REFRESH_STATUS
	mov	ah,al			; Start toggle state
	or	cx, cx
	jz	dn150n
	inc	cx			; Throwaway first toggle
;
dn150n:	in	al,PORTB		; Read system control port
	and	al,REFRESH_STATUS	; Refresh toggles 15.085 microseconds
	cmp	ah,al
	je	dn150n			; Wait for state change
;
	mov	ah,al			; Update with new state
	dec	cx
	jnz	dn150n
;
        pop     cx
        pop     ax
        ret
delay1ms     ENDP

;----------------------------------------------------------------------------
;       delay1_4ms - Delay for 1/4 millasecond.
;		    1mS = 1000us
;       Entry:
;         None
;       Exit:
;	  None
;
;       Modified:
;         None
;
PORTB			EQU	061h
  REFRESH_STATUS	EQU	010h		; Refresh signal status
delay1_4ms PROC NEAR
        push    ax 
        push    cx
        mov     cx, 16			; close enough.
	in	al,PORTB
	and	al,REFRESH_STATUS
	mov	ah,al			; Start toggle state
	or	cx, cx
	jz	@f
	inc	cx			; Throwaway first toggle
;
@@:	
	in	al,PORTB		; Read system control port
	and	al,REFRESH_STATUS	; Refresh toggles 15.085 microseconds
	cmp	ah,al
	je	@b			; Wait for state change
;
	mov	ah,al			; Update with new state
	dec	cx
	jnz	@b
;
        pop     cx
        pop     ax
        ret
delay1_4ms     ENDP
flopper_bottom:

